Attribute VB_Name = "MContext"
Attribute VB_Ext_KEY = "RVB_UniqueId" ,"390351020307"
Option Explicit

' Module shared by the business and data tiers

' Declaration for Win32 API function
Private Declare Function GetComputerNameAPI Lib "kernel32" Alias "GetComputerNameA" (ByVal lpBuffer As String, nSize As Long) As Long

Public Sub CtxRaiseError(ModuleName As String, MethodName As String)
' Change the transaction state, and raise an error.

  ' Set the default to disable transaction. Now unless someone
  ' does a SetComplete the transaction will abort.
  ' This is just like calling setabort, but has it doesn't destroy
  ' the Err object if we are in a transaction.
  CtxDisableCommit
  
  ' Log the error to the event for later use.
  LogError Err.Number, GetErrSourceDescription(ModuleName, _
    MethodName), Err.Description, "from CtxRaiseError"
  
  ' Raise an error to indate there was a problem.
  ' This will indicate that no one should do a SetComplete
  ' unless they can handle this error.
  '
  ' The following MsgBox code is for debugging purposes.
  ' It will permit a graceful way of acknowledging server side
  ' errors and enabling the client application to work after
  ' such an error. If you want to have the server side code
  ' terminate with a runtime error, comment out that and the
  ' Err.Clear and uncomment the Err.Raise statement.
  MsgBox "Error # " & Err.Number & vbCrLf & _
    GetErrSourceDescription(ModuleName, MethodName) & _
    vbCrLf & Err.Description
  Err.Clear
End Sub

Public Sub RaiseError(ModuleName As String, MethodName As String)
' Raise an error without changing the transaction state.

  ' Log the error to the event for later use.
  LogError Err.Number, GetErrSourceDescription(ModuleName, _
    MethodName), Err.Description, "From RaiseError"
  
  ' The following MsgBox code is for debugging purposes.
  ' It will permit a graceful way of acknowledging server side
  ' errors and enabling the client application to work after
  ' such an error. If you want to have the server side code
  ' terminate with a runtime error, comment out that and the
  ' Err.Clear and uncomment the Err.Raise statement.
  MsgBox "Error # " & Err.Number & vbCrLf & _
    GetErrSourceDescription(ModuleName, MethodName) & _
    vbCrLf & Err.Description
  Err.Clear
End Sub

Private Function GetComputerName() As String
' Set or retrieve the name of the computer.

  Dim strBuffer As String
  Dim lngLen As Long

  ' Prepare a buffer for the return value
  strBuffer = Space$(255 + 1)
  lngLen = Len(strBuffer)
  If CBool(GetComputerNameAPI(strBuffer, lngLen)) Then
    GetComputerName = Left$(strBuffer, lngLen)
  Else
    GetComputerName = ""
  End If
End Function

Function GetErrSourceDescription(ModuleName As String, MethodName As String) As String
' Returns an error message like:  "[Bank_DB.RangerZeroCurve]
' VerifyUser [on TOWW5391 version 5.21.176]"

  GetErrSourceDescription = Err.Source & vbNewLine & "<br>" & _
    "[" & ModuleName & "]  " & MethodName & " [on " & _
    GetComputerName() & " version " & GetVersionNumber() & "]"
End Function

Function GetVersionNumber() As String
' Returns the current DLL version number

  GetVersionNumber = App.Major & "." & App.Minor & "." & _
    App.Revision
End Function

Private Sub LogError(ErrorNumber As Long, ErrorSource As String, _
  ErrorDescription As String, Optional Notes As String = "")
' This procedure logs an error to the system application log

  Dim objErrorLogger As Bank_DB.Logger
  Dim blnError As Boolean
  
  Set objErrorLogger = CtxCreateObject("Bank_DB.Logger")
  blnError = objErrorLogger.LogError(ErrorNumber, ErrorSource, _
    ErrorDescription, Notes)
  If Not blnError Then
    App.LogEvent vbNewLine & "An error has occured in this application," _
      & "could not log to database." & vbNewLine & "Error Number:" _
      & ErrorNumber & vbNewLine & vbNewLine & "Description:" _
      & vbNewLine & ErrorDescription & vbNewLine & vbNewLine _
      & "Source: " & vbNewLine & ErrorSource _
      & vbNewLine & vbNewLine & "Notes: " & vbNewLine & Notes
  End If
  
  CtxSetComplete
  Set objErrorLogger = Nothing
End Sub

Public Sub CtxSetAbort()
' If the context exists, abort the transaction
  Dim objObjectContext As ObjectContext

  Set objObjectContext = GetObjectContext
  
  If Not (objObjectContext Is Nothing) Then
    objObjectContext.SetAbort
  End If
  
  Set objObjectContext = Nothing
End Sub

Public Sub CtxSetComplete()
' If the context exists, complete the transaction
  Dim objObjectContext As ObjectContext

  Set objObjectContext = GetObjectContext
  
  If Not (objObjectContext Is Nothing) Then
    objObjectContext.SetComplete
  End If
  
  Set objObjectContext = Nothing
End Sub

Public Sub CtxDisableCommit()
' If the context exists, disable committing it
  Dim objObjectContext As ObjectContext

  Set objObjectContext = GetObjectContext
  
  If Not (objObjectContext Is Nothing) Then
    objObjectContext.DisableCommit
  End If
    
  Set objObjectContext = Nothing
End Sub

Public Function CtxCreateObject(ByVal ProgID As String) As Object
' Create a transaction context with the ProgID string
  Dim objObjectContext As ObjectContext

  Set objObjectContext = GetObjectContext
  
  If Not (objObjectContext Is Nothing) Then
    Set CtxCreateObject = objObjectContext.CreateInstance(ProgID)
    Set objObjectContext = Nothing
  Else
    Set CtxCreateObject = CreateObject(ProgID)
  End If
End Function
