VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 1  'NoTransaction
END
Attribute VB_Name = "Admin"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Class module in business tier

Const MODULE_NAME As String = "Bank_Bus.Admin"

Public Property Get GetAccountType(ByVal BankID As Integer, _
  ByVal CardID As Long) As Boolean
' See if the CardID account is an Administrator account
' True is an Admin, False is not
  Dim objBank As Bank_DB.Admin
  Dim recAccounts As ADODB.Recordset

  On Error GoTo ErrorHandler
  GetAccountType = False
  
  ' Create a transaction context for the DB call
  Set objBank = CtxCreateObject("Bank_DB.Admin")
  
  'Passthrough call to data tier
  Set recAccounts = objBank.GetAccountType(BankID, CardID)
  
  CtxSetComplete 'Commit the transaction
  
  Do While Not recAccounts.EOF
    If Trim$(recAccounts!AccountTypeDesc) = "Administrator" Then
      GetAccountType = True
    End If
    
    recAccounts.MoveNext
  Loop

Clean_Up:
  Set recAccounts = Nothing
  Set objBank = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "GetAccountType(" & CStr(BankID) & _
    ", " & CStr(CardID) & ")"
  GoTo Clean_Up
End Property

Public Property Get GetAccountTypes(ByVal BankID As Integer) _
  As ADODB.Recordset
  ' Get the account types.

  Dim objBank As Bank_DB.Admin
  
  On Error GoTo ErrorHandler

  Set objBank = CtxCreateObject("Bank_DB.Admin")
  
  'Passthrough call to data tier
  Set GetAccountTypes = objBank.GetAccountTypes(BankID)
  
  CtxSetComplete
  
Clean_Up:
  Set objBank = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "GetAccountTypes(" & CStr(BankID) & ")"
  GoTo Clean_Up
End Property

Public Property Get AddBank(ByVal BankID As Integer, _
  ByVal NewBankID As Integer, ByVal Description As String, _
  ByVal DSN As String) As Boolean
  
  Dim objBank As Bank_DB.Admin

  On Error GoTo ErrorHandler
  AddBank = False
  
  ' Create a transaction context for the DB call
  Set objBank = CtxCreateObject("Bank_DB.Admin")
  
  'Passthrough call to data tier
  AddBank = objBank.AddBank(BankID, NewBankID, Description, DSN)
  
  CtxSetComplete 'Commit the transaction

Clean_Up:
  Set objBank = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "AddBank(" & CStr(NewBankID) & _
    ", " & Description & "," & DSN & ")"
  GoTo Clean_Up
End Property

Public Property Get EditBank(ByVal BankID As Integer, _
  ByVal ChangeBankID As Integer, ByVal Description As String, _
  ByVal DSN As String) As Boolean
  
  Dim objBank As Bank_DB.Admin

  On Error GoTo ErrorHandler
  EditBank = False
  
  ' Create a transaction context for the DB call
  Set objBank = CtxCreateObject("Bank_DB.Admin")
  
  'Passthrough call to data tier
  EditBank = objBank.EditBank(BankID, ChangeBankID, Description, DSN)
  
  CtxSetComplete 'Commit the transaction

Clean_Up:
  Set objBank = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "EditBank(" & CStr(ChangeBankID) & _
    ", " & Description & "," & DSN & ")"
  GoTo Clean_Up
End Property

Public Property Get DeleteBank(ByVal BankID As Integer, _
  ByVal ChangeBankID As Integer) As Boolean
  
  Dim objBank As Bank_DB.Admin

  On Error GoTo ErrorHandler
  DeleteBank = False
  
  ' Create a transaction context for the DB call
  Set objBank = CtxCreateObject("Bank_DB.Admin")
  
  'Passthrough call to data tier
  DeleteBank = objBank.DeleteBank(BankID, ChangeBankID)
  
  CtxSetComplete 'Commit the transaction

Clean_Up:
  Set objBank = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "DeleteBank(" & CStr(ChangeBankID) & ")"
  GoTo Clean_Up
End Property

Public Property Get VerifyCard(ByVal BankID As Integer, _
  ByVal CardID As Long) As Boolean

  Dim objBank As Bank_DB.Admin
  Dim lngReturn As Long

  On Error GoTo ErrorHandler
  VerifyCard = False
  
  ' Create a transaction context for the DB call
  Set objBank = CtxCreateObject("Bank_DB.Admin")
  
  'Passthrough call to data tier
  lngReturn = objBank.VerifyCard(BankID, CardID)
  If lngReturn <> 0 Then
    VerifyCard = True
  End If
  
  CtxSetComplete 'Commit the transaction

Clean_Up:
  Set objBank = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "VerifyCard(" & CStr(CardID) & ")"
  GoTo Clean_Up
End Property

Public Property Get DeleteCard(ByVal BankID As Integer, _
  ByVal CardID As Long) As Boolean
  
  Dim objBank As Bank_DB.Admin

  On Error GoTo ErrorHandler
  DeleteCard = False
  
  ' Create a transaction context for the DB call
  Set objBank = CtxCreateObject("Bank_DB.Admin")
  
  'Passthrough call to data tier
  DeleteCard = objBank.DeleteCard(BankID, CardID)
  
  CtxSetComplete 'Commit the transaction

Clean_Up:
  Set objBank = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "DeleteCard(" & CStr(CardID) & ")"
  GoTo Clean_Up
End Property

Public Property Get AddCard(ByVal BankID As Integer, _
  ByVal AccountType As Integer, ByVal InitialBalance As Currency, _
  ByVal DailyLimit As Currency, ByVal OverDraft As Currency, _
  ByRef CardID As String, ByRef Password As String) As Boolean
  
  Dim objBank As Bank_DB.Admin
  Dim lngCardID As Long
  Dim lngAccountID As Long
  Dim strPassword As String

  On Error GoTo ErrorHandler
  AddCard = False
  
  lngCardID = GenerateCardID(BankID)
  CardID = CStr(lngCardID)
  
  lngAccountID = GenerateAccountID(BankID)
  
  strPassword = GeneratePassword(lngAccountID)
  Password = strPassword
  
  ' Create a transaction context for the DB call
  Set objBank = CtxCreateObject("Bank_DB.Admin")
  
  'Passthrough call to data tier
  AddCard = objBank.AddCard(BankID, AccountType, InitialBalance, _
  DailyLimit, OverDraft, strPassword, lngCardID, lngAccountID)
  
  CtxSetComplete 'Commit the transaction

Clean_Up:
  Set objBank = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "AddCard(" & CStr(BankID) & ")"
  GoTo Clean_Up
End Property

Private Function GeneratePassword(ByVal lngAccountID As Long) _
  As String
  ' This simple password is created by combining the word
  ' "secret" with the rightmost 4 digits of the AccountID
  GeneratePassword = "secret" & Right$(CStr(lngAccountID), 4)
End Function

Private Function GenerateCardID(ByVal BankID As Integer) As Long
' Generate a new CardID, which will be the previous last CardID + 1
  Dim objBank As Bank_DB.Admin

  On Error GoTo ErrorHandler
  
  ' Create a transaction context for the DB call
  Set objBank = CtxCreateObject("Bank_DB.Admin")
  
  'Passthrough call to data tier
  GenerateCardID = objBank.GetLastCardId(BankID)
  
  CtxSetComplete 'Commit the transaction
  
  GenerateCardID = GenerateCardID + 1

Clean_Up:
  Set objBank = Nothing
  Exit Function

ErrorHandler:
  RaiseError MODULE_NAME, "GenerateCardID(" & CStr(BankID) & ")"
  GoTo Clean_Up
End Function

Private Function GenerateAccountID(ByVal BankID As Integer) As Long
  ' Generate a new AccountID, which will be the previous
  ' last AccountID + 1
  
  Dim objBank As Bank_DB.Admin

  On Error GoTo ErrorHandler
  
  ' Create a transaction context for the DB call
  Set objBank = CtxCreateObject("Bank_DB.Admin")
  
  'Passthrough call to data tier
  GenerateAccountID = objBank.GetLastAccountId(BankID)
  
  CtxSetComplete 'Commit the transaction
  
  GenerateAccountID = GenerateAccountID + 1

Clean_Up:
  Set objBank = Nothing
  Exit Function

ErrorHandler:
  RaiseError MODULE_NAME, "GenerateAccountID(" & CStr(BankID) & ")"
  GoTo Clean_Up
End Function

