VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 4  'RequiresNewTransaction
END
Attribute VB_Name = "Transfer"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Class module in business tier

Const MODULE_NAME As String = "Bank_Bus.Transfer"

' NOTE: THESE VALUES CORRESPOND
' TO ENTRIES IN A DATABASE TABLE
' THIS ENUM AND THE TABLE MUST BE
' KEPT SYNCHRONIZED.
' SEE BANK_DB SOURCE FOR MORE DETAILS.
Public Enum ETransactionType
  txTransfer = Bank_DB.txTransfer
  txBillPayment = Bank_DB.txBillPayment
End Enum

' Return values enumeration for funds transfers
Public Enum ETransactionStatus
  txsSuccess = 0
  txsDailyLimitExceeded = 1
  txsNotEnoughFunds = 2
  txsUnknownError = 3
End Enum

Public Property Get TransferFunds(ByVal FromBankId As Long, _
  ByVal FromAccountId As Long, ByVal ToBankId As Long, _
  ByVal ToAccountId As Long, ByVal Amount As Currency, _
  ByVal enuTransactionType As ETransactionType, _
  ByVal Description As String) As ETransactionStatus
' Transfers funds from a source to a destination account

  Dim objFromAccount As Bank_DB.Account
  Dim objToAccount As Bank_DB.Account
  Dim ctxObjectContext As ObjectContext
  Dim enuTransactionStatus As ETransactionStatus
  Dim intCount As Integer
  
  On Error GoTo ErrorHandler

  ' Get a context object
  Set ctxObjectContext = GetObjectContext
  
  ' Verify that selected account exists before debiting
  ' the source account
  Set objToAccount = New Bank_DB.Account
  'Passthrough call to data tier
  intCount = objToAccount.VerifyAccount(ToBankId, ToAccountId)
  
  If intCount <> 0 Then
    ' If the account exists
    Set ctxObjectContext = GetObjectContext
    
    Set objFromAccount = New Bank_DB.Account
  
    'Passthrough call to data tier
    ' Debit source account
    enuTransactionStatus = objFromAccount.DecrementBalance( _
      FromBankId, FromAccountId, Amount, CInt(enuTransactionType), _
      Description, CStr(ToBankId) & "-" & CStr(ToAccountId))
  
    If enuTransactionStatus = txsSuccess Then
      Set objToAccount = CtxCreateObject("Bank_DB.Account")
  
      'Passthrough call to data tier
      ' Increment destination account
      objToAccount.IncrementBalance ToBankId, _
        ToAccountId, Amount, CInt(enuTransactionType), _
        Description, CStr(FromBankId) & "-" & CStr(FromAccountId)
    Else
      ' If the debit failed abort the transaction
      If Not (ctxObjectContext Is Nothing) Then
        ctxObjectContext.SetAbort
      End If
    End If
  Else
    enuTransactionStatus = txsUnknownError
  End If
  
  TransferFunds = enuTransactionStatus
  
Clean_Up:
  Set objFromAccount = Nothing
  Set objToAccount = Nothing
  Set ctxObjectContext = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "TransferFunds(" & CStr(FromBankId) & ", " _
    & CStr(FromAccountId) & ", " & CStr(ToBankId) & ", " & _
    CStr(ToAccountId) & ", " & CStr(Amount) & ", " & _
    CStr(enuTransactionType) & ", " & Description & ")"
  
  If Not (ctxObjectContext Is Nothing) Then
    ctxObjectContext.SetAbort
  End If
  
  TransferFunds = txsUnknownError
  GoTo Clean_Up
End Property

