VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 2  'RequiresTransaction
END
Attribute VB_Name = "Account"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Class module in data tier

Const MODULE_NAME As String = "Bank_DB.Account"

' Sizes for strings in tables
Const TXDESCRIPTION_SIZE As Integer = 40
Const COUNTERPARTYREF_SIZE As Integer = 16

'These must match the enumerated values in Bank_Bus.Transfer
Const txsSuccess = 0
Const txsUnknownError = 3

Private m_objDBHelper As DBHelper

Private Sub Class_Initialize()
' Initialize the helper object to run stored procedures
   Set m_objDBHelper = CreateObject("Bank_DB.DBHelper")
End Sub

Private Sub Class_Terminate()
' Destroy the helper object
   Set m_objDBHelper = Nothing
End Sub

Public Sub IncrementBalance(ByVal BankID As Integer, _
  ByVal AccountID As Long, ByVal Amount As Currency, _
  intTransactionType As Integer, Description As String, _
  CounterParty As String)
' Increment an account balance and log the transaction

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  If strDSN <> "" Then
    ' If the bank exists run the stored procedure
    m_objDBHelper.RunSP strDSN, "sp_update_BankAccount_Inc", _
      Array("@AccountId_1", adInteger, adSizeInteger, AccountID), _
      Array("@TxAmount_2", adCurrency, adSizeCurrency, Amount), _
      Array("@TxType_3", adTinyInt, adSizeTinyInt, _
      intTransactionType), Array("@TxDescription_4", adChar, _
      TXDESCRIPTION_SIZE, Description), Array("@CounterpartyRef_5", _
      adChar, 16, CounterParty)
  
    CtxSetComplete
  End If

Clean_Up:
  Set objBankLocator = Nothing
  Exit Sub
  
ErrorHandler:
  RaiseError MODULE_NAME, "IncrementBalance"
  GoTo Clean_Up
End Sub

Public Function DecrementBalance(ByVal BankID As Integer, _
  ByVal AccountID As Long, ByVal Amount As Currency, _
  ByRef intTransactionType As Integer, ByRef Description As String, _
  ByRef CounterParty As String) As Integer
' Decrement an account balance and log the transaction

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  If strDSN <> "" Then
    ' If the bank exists run the stored procedure
    DecrementBalance = m_objDBHelper.RunSPReturnInteger( _
      strDSN, "sp_update_BankAccount_Dec", _
      Array("@AccountId_1", adInteger, adSizeInteger, _
      AccountID), Array("@TxAmount_2", adCurrency, _
      adSizeCurrency, Amount), Array("@TxType_3", _
      adTinyInt, adSizeTinyInt, intTransactionType), _
      Array("@TxDescription_4", adChar, TXDESCRIPTION_SIZE, _
      Description), Array("@CounterpartyRef_5", adChar, 16, _
      CounterParty))
    CtxSetComplete
  Else
    DecrementBalance = txsUnknownError
  End If
  
Clean_Up:
  Set objBankLocator = Nothing
  Exit Function

ErrorHandler:
  RaiseError MODULE_NAME, "DecrementBalance"
  DecrementBalance = txsUnknownError
  GoTo Clean_Up
End Function

Public Property Get VerifyAccount(ByVal BankID As Integer, _
  ByVal AccountID As Long) As Integer
' Verify that the selected account exists at the selected bank

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  If strDSN <> "" Then
    ' If the bank exists run the stored procedure.
    ' Returns the count of accounts that have this AccountID
    VerifyAccount = m_objDBHelper.RunSPReturnInteger( _
      strDSN, "sp_verify_BankAccount_By_CardId", _
      Array("@AccountId_1", adInteger, adSizeInteger, _
      AccountID))
    CtxSetComplete
  Else
    VerifyAccount = 0 'Set to no accounts
  End If
  
Clean_Up:
  Set objBankLocator = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "VerifyAccount"
  VerifyAccount = 0 'Set to no accounts
  GoTo Clean_Up
End Property

