VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 1  'NoTransaction
END
Attribute VB_Name = "DBHelper"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Class module in data tier

' Runs procedures stored in the database

Const MODULE_NAME As String = "Bank_DB.DBHelper"

Function RunSPReturnRS(ByVal ConnectionString As String, _
  ByVal StoredProcedureName As String, ParamArray ParameterArray() _
  As Variant) As ADODB.Recordset
' Run a stored procedure and return a recordset

  Dim rst As ADODB.Recordset
  Dim cmd As ADODB.Command
  
  On Error GoTo ErrorHandler

  ' Create the ADO objects
  Set rst = New ADODB.Recordset
  Set cmd = New ADODB.Command
  
  ' Init the ADO objects
  With cmd
    .ActiveConnection = ConnectionString
    .CommandText = StoredProcedureName
    .CommandType = adCmdStoredProc
  End With
  rst.CursorLocation = adUseClient

  ' Init the stored procedure parameters
  CollectParams cmd, ParameterArray
  
  ' Execute the stored procedure for readonly
  rst.Open cmd, , adOpenForwardOnly, adLockReadOnly
  
Clean_Up:
  ' Disconnect the recordset
  Set cmd.ActiveConnection = Nothing
  Set cmd = Nothing
  Set rst.ActiveConnection = Nothing
  
  ' Return the recordset
  Set RunSPReturnRS = rst
  Set rst = Nothing
  Exit Function
  
ErrorHandler:
  Set rst = Nothing
  RaiseError MODULE_NAME, "RunSPReturnRS(" _
    & StoredProcedureName & ", ...)"
  GoTo Clean_Up
End Function

Function RunSP(ByVal ConnectionString As String, _
  ByVal StoredProcedureName As String, _
  ParamArray ParameterArray() As Variant)
' Run a stored procedure with no return value

  Dim cmd As ADODB.Command
  
  On Error GoTo ErrorHandler

  ' Create the ADO objects
  Set cmd = New ADODB.Command
  
  ' Init the ADO objects
  With cmd
    .ActiveConnection = ConnectionString
    .CommandText = StoredProcedureName
    .CommandType = adCmdStoredProc
  End With
  
  ' Init the stored procedure parameters
  CollectParams cmd, ParameterArray
  
  ' Execute the stored procedure without returning a recordset
  cmd.Execute , , adExecuteNoRecords

  ' Disconnect the recordset and clean up
Clean_Up:
  Set cmd.ActiveConnection = Nothing
  Set cmd = Nothing
  
  Exit Function

ErrorHandler:
  RaiseError MODULE_NAME, "RunSP(" & StoredProcedureName & _
    ", ...)"
  GoTo Clean_Up
End Function

Function RunSPReturnInteger(ByVal ConnectionString As String, _
  ByVal StoredProcedureName As String, _
  ParamArray ParameterArray() As Variant) As Long
' Run a stored procedure and return a value
' NOTE:  The return type is a Long, which corresponds to the
' adInteger type selected for @ReturnValue

  Dim cmd As ADODB.Command
  
  On Error GoTo ErrorHandler

  ' Create the ADO objects
  Set cmd = New ADODB.Command
  
  ' Init the ADO objects
  With cmd
    .ActiveConnection = ConnectionString
    .CommandText = StoredProcedureName
    .CommandType = adCmdStoredProc
  End With
  
  CollectParams cmd, ParameterArray
  
  ' Assume the last parameter is outgoing
  cmd.Parameters.Append cmd.CreateParameter("@ReturnValue", _
    adInteger, adParamOutput, adSizeInteger)
  
  ' Execute without a resulting recordset
  cmd.Execute , , adExecuteNoRecords
  ' Pull out the "return value" parameter
  RunSPReturnInteger = cmd.Parameters("@ReturnValue").Value
  
  ' Disconnect the recordset, and clean up
Clean_Up:
  Set cmd.ActiveConnection = Nothing
  Set cmd = Nothing
  
  Exit Function

ErrorHandler:
  RunSPReturnInteger = -1
  Set cmd = Nothing
  RaiseError MODULE_NAME, "RunSPReturnInteger(" & _
    StoredProcedureName & ", ...)"
  GoTo Clean_Up
End Function

Friend Sub CollectParams(ByRef cmd As ADODB.Command, _
  ParamArray ParamaterArray() As Variant)
' Convert a Variant array into a set of parameters for the
' stored procedure

  Dim vntParameterArray As Variant
  Dim vntParameter As Variant
  Dim intParameterCnt As Integer
  Dim intParameterArrayUBound As Integer

  vntParameterArray = ParamaterArray(0)
  intParameterArrayUBound = UBound(vntParameterArray)
  
  ' Loop through the array of parameters and collect
  ' the 4 required arguments for each parameter -
  ' Name, Data Type, Size, Value
  For intParameterCnt = LBound(vntParameterArray) _
    To intParameterArrayUBound
    If UBound(vntParameterArray(intParameterCnt)) - _
      LBound(vntParameterArray(intParameterCnt)) = 3 Then
      ' Check for nulls.
      If VarType(vntParameterArray(intParameterCnt)(3)) = _
        vbString Then
        vntParameter = IIf(vntParameterArray(intParameterCnt)(3) _
          = "", Null, vntParameterArray(intParameterCnt)(3))
      Else
        vntParameter = vntParameterArray(intParameterCnt)(3)
      End If
      cmd.Parameters.Append cmd.CreateParameter( _
        vntParameterArray(intParameterCnt)(0), _
        vntParameterArray(intParameterCnt)(1), adParamInput, _
        vntParameterArray(intParameterCnt)(2), vntParameter)
    Else
      CtxRaiseError MODULE_NAME, "CollectParams(...): " _
        & "incorrect # of parameters"
    End If
  Next intParameterCnt
End Sub
