VERSION 5.00
Begin VB.UserControl ctlCustCombo 
   ClientHeight    =   3600
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   4800
   PropertyPages   =   "ctlCustCombo.ctx"
   ScaleHeight     =   3600
   ScaleWidth      =   4800
   Begin VB.ComboBox cboCustomer 
      Height          =   315
      Left            =   360
      Style           =   2  'Dropdown List
      TabIndex        =   0
      Top             =   1080
      Width           =   2535
   End
End
Attribute VB_Name = "ctlCustCombo"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Declare two variable to deal with the database and
' customer query results.
Private m_dbDatabase As Database
Private m_recCustomers As Recordset

' Need two member variables to hold the limit strings
Private m_strRegion As String
Private m_strCountry As String

' Need an array to hold the customer ID's of the customers
' in the list
Private m_strCustID() As String

Public Event CustomerSelected(ByVal strCustomerID As String, _
    ByVal strCustomerName As String)

Private Sub cboCustomer_Click()
' When an item in the combo box gets clicked we need to
' reposition the recordset at the selected record.
    If cboCustomer.ListIndex > -1 Then
        m_recCustomers.FindFirst "CustomerID = '" & _
            m_strCustID(cboCustomer.ListIndex) & "'"
        RaiseEvent CustomerSelected(m_recCustomers!CustomerID, _
            m_recCustomers!CompanyName)
    End If
    
End Sub

Private Sub UserControl_AmbientChanged(PropertyName As String)
    If UCase$(PropertyName) = "BACKCOLOR" Then
        cboCustomer.BackColor = Ambient.BackColor
    End If
    
End Sub

Private Sub UserControl_Initialize()
' Connects to the database, and grabs the initial query.

    ' First, open the database itself.
    Set m_dbDatabase = Workspaces(0).OpenDatabase _
        (App.Path & "\Nwind.mdb")
        
End Sub

Private Sub UserControl_ReadProperties(PropBag As PropertyBag)

    With PropBag
        m_strCountry = .ReadProperty("LimitCountry", "")
        m_strRegion = .ReadProperty("LimitRegion", "")
    End With
    
    Rebuild_Query
    
End Sub

Private Sub UserControl_Resize()
    cboCustomer.Move 0, 0, ScaleWidth
    Height = cboCustomer.Height
End Sub

Private Sub UserControl_Terminate()
' Closes down the database and recordset connections

    On Error Resume Next
    ' Get rid of the customers query
    m_recCustomers.Close
    
    ' Then, lose the database connection
    m_dbDatabase.Close
    
End Sub

Public Property Get CustID() As String
' Returns the CustID field from the currently selected
' customer. If there is no customer selected, then "" is
' returned.

    ' Set up an error handler in case there is no current
    ' record to get the ID from
    On Error GoTo Get_CustID_Error
        
    CustID = m_recCustomers!CustID
    Exit Property
    
Get_CustID_Error:       ' This error handler triggered when no
                        ' current record.
    CustID = ""
    Exit Property
    
End Property

Public Property Get CustomerName() As String
' Returns the CompanyName field from the currently selected
' customer. If there is no customer selected, then "" is
' returned.
    
    On Error GoTo Get_CustomerName_Error
    
    CustomerName = m_recCustomers!CompanyName
    Exit Property
    
Get_CustomerName_Error:

    CustomerName = ""
    Exit Property
    
End Property

Public Property Get CustomerCount() As Integer
'Returns a count of the number of customers being
' managed by the combo box in the control.

    CustomerCount = cboCustomer.ListCount
    
End Property

Public Property Let LimitRegion(ByVal strRegion As String)
' Sets up the region to limit the customers to

    If strRegion <> m_strRegion Then    'if a new limit has been
                                        ' assigned then rebuild the query
        m_strRegion = strRegion
        Rebuild_Query
    End If
    
End Property

Public Property Get LimitRegion() As String
' returns the current LimitRegion setting

    LimitRegion = m_strRegion
    
End Property

Public Property Let LimitCountry(ByVal strCountry As String)
' Sets up the country to limit the customers to

If strCountry <> m_strCountry Then
    m_strCountry = strCountry
    Rebuild_Query
End If

PropertyChanged "LimitCountry"
End Property

Public Property Get LimitCountry() As String
' Returns the current LimitCountry setting

    LimitCountry = m_strCountry
    
End Property

Private Sub Rebuild_Query()
' Routine to rebuild the query, taking into account the
' current applied limits

Dim strQuery As String
Dim strWhere As String

If Ambient.UserMode = False Then Exit Sub

' First set up the constant part of the query
strQuery = "SELECT CustomerID, CompanyName, City FROM Customers"

' Next take a look to see if we need to add on anything
' to limit the regions
If m_strRegion <> "" Then strWhere = "Region = '" & m_strRegion & "'"

' ... then, take a look to see if we need to deal with
' a country
If m_strCountry <> "" Then

    ' If the WHERE clause already has something in it,
    ' we need to tack the word AND onto the end of it
    If strWhere <> "" Then strWhere = strWhere & " AND "
    
        strWhere = strWhere & "Country = '" & m_strCountry & "'"
    
    End If
    
' Finally, if the WHERE clause has anything at all in it
' we need to tack it on to the end of our query
If strWhere <> "" Then strQuery = strQuery & " WHERE " & strWhere

    ' All that remains is to re-issue the query
    On Error Resume Next
    m_recCustomers.Close
    On Error GoTo 0
    
    Set m_recCustomers = m_dbDatabase.OpenRecordset _
        (strQuery, dbOpenSnapshot)
        
    Load_Data
        
End Sub

Private Sub Load_Data()
' Loads the data from the recordset into the combo box

' First, clear out any exsisting data in the combo.
cboCustomer.Clear
ReDim m_strCustID(0)

' Next, start a loop through the data and load it into
' the combo box (remember, there could be no data to
' load, so an error handler should be used to catch that)
On Error GoTo Load_Data_Error
m_recCustomers.MoveFirst
On Error GoTo 0

Do While Not m_recCustomers.EOF

    cboCustomer.AddItem m_recCustomers!CompanyName & ", " & _
        m_recCustomers!City
    ReDim Preserve m_strCustID(cboCustomer.ListCount)
    m_strCustID(cboCustomer.ListCount - 1) = _
        m_recCustomers!CustomerID
    m_recCustomers.MoveNext
    
Loop

Load_Data_Error:
    Exit Sub
    
End Sub

Private Sub UserControl_WriteProperties(PropBag As PropertyBag)

    With PropBag
    
        .WriteProperty "LimitCountry", LimitCountry
        .WriteProperty "LimitRegion", LimitRegion
    
    End With
    
End Sub

Public Property Get TextFont() As StdFont
    Set TextFont = cboCustomer.Font
End Property

Public Property Set TextFont(ByVal TheFont As StdFont)
    Set cboCustomer.Font = TheFont
End Property

Public Property Get ComboTextColor() As OLE_COLOR
    ComboTextColor = cboCustomer.ForeColor
End Property

Public Property Let ComboTextColor(ByVal NewColor As OLE_COLOR)
    cboCustomer.ForeColor = NewColor
End Property

Public Property Get ComboBackColor() As OLE_COLOR
    ComboBackColor = cboCustomer.BackColor
End Property

Public Property Let ComboBackColor(ByVal NewColor As OLE_COLOR)
    cboCustomer.BackColor = NewColor
End Property

Public Property Get DB() As Database
    Set DB = m_dbDatabase
End Property
